/*
 * Copyright (C) 2013 Christopher Burke <krslynx@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package org.xproduct.server.core.helpers;

import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Map;
import org.json.simple.parser.JSONParser;
import org.json.simple.parser.ParseException;

/**
 *
 * @author Christopher Burke <krslynx@gmail.com>
 */
public class Converters {

    /**
     * Converts a JSON String to a java.util.Map which, unfortunately, does not
     * preserve the order of the data in the String however allows for easy
     * manipulation of the data.
     *
     * @param jsonString String data to be converted
     * @return the Map
     * @throws ParseException
     */
    public static Map jsonStringToMap(String jsonString) throws ParseException {
        JSONParser parser = new JSONParser();
        Object obj = parser.parse(jsonString);
        return (Map) obj;
    }

    /**
     * Converts a JSON String to a Object
     *
     * @param jsonString String data to be converted
     * @return the Object
     * @throws ParseException
     */
    public static Object jsonStringToObject(String jsonString) throws ParseException {
        JSONParser parser = new JSONParser();
        Object obj = parser.parse(jsonString);
        return obj;
    }

    /**
     * @return milis since unix timestamp epoch origin
     */
    public static long getTimeInMilis() {
        return Calendar.getInstance().getTimeInMillis();
    }

    /**
     * Converts a time (in long format) to a 32bit unsigned integer to match the
     * output returned from Google closures goog.now() function.
     *
     * @return time as uint32 to match goog.now()
     */
    public static long timeAsUInt32() {
        return longToUInt32(getTimeInMilis());
    }

    /**
     * Converts a long to an 32bit unsigned integer.
     *
     * @param value long value to convert
     * @return integer as 32bit unsigned
     */
    public static long longToUInt32(long value) {
        return value & 0x00000000FFFFFFFFL;
    }

    /**
     * JSON converters often convert integers to double, this converts that
     * double value (at the specified offset) to an integer.
     *
     * @param object object array to retrieve data
     * @param offset offset to retrieve data from
     * @return integer value
     * @deprecated 
     */
    public static int getIntegerAtOffset(Object[] object, int offset) {
        Double dbl = ((Double) object[offset]);
        return dbl.intValue();
    }

    /**
     * 
     * @param object
     * @param offset
     * @return 
     * @deprecated 
     */
    public static long getLongAtOffset(Object[] object, int offset) {
        return (Long) object[offset];
    }

    public static String getString(int offset, Object[] incomingObject) {
        return incomingObject[offset].toString();
    }

    public static int getInteger(int offset, Object[] incomingObject) {
        String intString = incomingObject[offset].toString();
        if(intString.endsWith(".0")) {
            intString = intString.replace(".0", "");
        } else {
            intString = intString.replace(".", "");
        }
        return new Integer(intString);
    }

    public static long getLong(int offset, Object[] incomingObject) {
        return ((Double) Double.parseDouble(incomingObject[offset].toString())).longValue();
    }

    public static BigDecimal getBigDecimal(int offset, Object[] incomingObject) {
        return new BigDecimal(incomingObject[offset].toString());
    }

    public static double getDouble(int offset, Object[] incomingObject) {
        String dblString = incomingObject[offset].toString();
        if(dblString.endsWith(".0")) {
            dblString = dblString.replace(".0", "");
        } else {
            dblString = dblString.replace(".", "");
        }
        return new Double(dblString);
    }

    public static boolean getBooleanFromInteger(int offset, Object[] incomingObject) {
        boolean returnBoolean = false;

        String dataString = incomingObject[offset].toString();
        if(dataString.endsWith(".0")) {
            dataString = dataString.replace(".0", "");
        } else {
            dataString = dataString.replace(".", "");
        }
        
        if (dataString.equalsIgnoreCase("1")) {
            returnBoolean = true;
        }

        return returnBoolean;
    }
}
