/*
 * Copyright (C) 2013 Christopher Burke <krslynx@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package org.xproduct.server.core.net.controllers;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.vertx.java.core.http.ServerWebSocket;
import org.xproduct.server.core.Game;
import org.xproduct.server.core.exceptions.GameProtocolException;
import org.xproduct.server.core.model.Player;
import org.xproduct.server.core.net.PacketController;
import org.xproduct.server.core.net.packets.core.c2s.C2SPrizeCollected;
import org.xproduct.server.core.net.packets.core.s2c.S2CPrizeCollected;

/**
 *
 * @author Christopher Burke <krslynx@gmail.com>
 */
public class PrizeCollectedController extends PacketController {

    /**
     * The logger for the class
     */
    private Logger log = LoggerFactory.getLogger(PrizeCollectedController.class);

    public PrizeCollectedController(Object[] incomingObject, Game gameServer, ServerWebSocket webSocket) {
        super(incomingObject, gameServer, webSocket);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void process() throws GameProtocolException {
        Player player = getPlayer();
        if(player == null) {
            return;
        }
        C2SPrizeCollected prizeCollectedIn = new C2SPrizeCollected(incomingObject);
        S2CPrizeCollected prizeCollectedOut = new S2CPrizeCollected(
                player.getSocialIdentifier().getId(), 
                prizeCollectedIn.getType(), 
                prizeCollectedIn.getX(), 
                prizeCollectedIn.getY());
        sendToAll(prizeCollectedOut.getPacket());
        
    }
    
}
