/*
 * Copyright (C) 2013 Christopher Burke <krslynx@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package org.xproduct.server.core.net.controllers;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map.Entry;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.vertx.java.core.http.ServerWebSocket;
import org.xproduct.server.core.Game;
import org.xproduct.server.core.exceptions.GameProtocolException;
import org.xproduct.server.core.helpers.Converters;
import org.xproduct.server.core.model.Player;
import org.xproduct.server.core.net.PacketController;
import org.xproduct.server.core.net.packets.core.c2s.C2SStartGame;
import org.xproduct.server.core.net.packets.core.s2c.S2CPlayerEntered;
import org.xproduct.server.core.net.packets.core.s2c.S2CPrizeSeedUpdate;
import org.xproduct.server.core.net.packets.core.s2c.S2CScoreUpdate;

/**
 *
 * @author Christopher Burke <krslynx@gmail.com>
 */
public class StartGameController extends PacketController {

    /**
     * The logger for the class
     */
    private Logger log = LoggerFactory.getLogger(StartGameController.class);

    public StartGameController(Object[] incomingObject, Game gameServer, ServerWebSocket webSocket) {
        super(incomingObject, gameServer, webSocket);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void process() throws GameProtocolException {
        C2SStartGame c2sStartGame = new C2SStartGame(incomingObject);

        Player player = getPlayer();
        if (player == null) {
            return;
        }

        player.setShip(c2sStartGame.getShip());

        S2CScoreUpdate s2cScoreUpdate = new S2CScoreUpdate(
                player.getSocialIdentifier().getId(),
                player.getScore().getPoints().getTotalPoints(),
                player.getScore().getKills().getTotalKills(),
                player.getScore().getDeaths().getTotalDeaths());
        sendToAll(s2cScoreUpdate.getPacket());


        S2CPrizeSeedUpdate s2cPrizeSeedUpdate = new S2CPrizeSeedUpdate(
                1669142422L, // XXX: Get first clocksync time
                Converters.timeAsUInt32());
        sendToAll(s2cPrizeSeedUpdate.getPacket());


        S2CPlayerEntered s2cPlayerEntered = new S2CPlayerEntered(
                player.getSocialIdentifier().getId(),
                player.getHandle(),
                player.getTeam().getIdentifier(),
                player.isAlive(),
                player.getShip(),
                player.getBounty(),
                player.isPresentInArena());
        sendToAll(s2cPlayerEntered.getPacket());
        
        
        HashMap<ServerWebSocket, Player> playerMap = gameServer.getPlayerMap();
        Iterator<Entry<ServerWebSocket, Player>> playerMapIterator
                = playerMap.entrySet().iterator();
        
        while(playerMapIterator.hasNext()) {
            Entry<ServerWebSocket, Player> entry = playerMapIterator.next();
            
            if(entry.getValue() != player) {
                Player p = entry.getValue();
                
                S2CPlayerEntered _s2cPlayerEntered = new S2CPlayerEntered(
                        p.getSocialIdentifier().getId(),
                        p.getHandle(),
                        p.getTeam().getIdentifier(),
                        p.isAlive(),
                        p.getShip(),
                        p.getBounty(),
                        p.isPresentInArena());
                send(_s2cPlayerEntered.getPacket());

                S2CScoreUpdate _s2cScoreUpdate = new S2CScoreUpdate(
                        p.getSocialIdentifier().getId(),
                        p.getScore().getPoints().getTotalPoints(),
                        p.getScore().getKills().getTotalKills(),
                        p.getScore().getDeaths().getTotalDeaths());
                send(_s2cScoreUpdate.getPacket());
            }
        }
    }
}
