/*
 * Copyright (C) 2013 Christopher Burke <krslynx@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package org.xproduct.server.core.net.packets.core.c2s;

import java.math.BigDecimal;
import static org.xproduct.server.core.helpers.Converters.getBigDecimal;
import static org.xproduct.server.core.helpers.Converters.getBooleanFromInteger;
import static org.xproduct.server.core.helpers.Converters.getInteger;
import static org.xproduct.server.core.helpers.Converters.getLong;
import org.xproduct.server.core.net.packets.C2SPacketType;
import org.xproduct.server.core.net.packets.core.C2S;

/**
 *
 * @author Christopher Burke <krslynx@gmail.com>
 */
public class C2SPosition extends C2S {

    private long remoteTimeDelta;
    private BigDecimal direction;
    private BigDecimal posX;
    private BigDecimal posY;
    private BigDecimal velX;
    private BigDecimal velY;
    private int isSafe;
    private int projectileType;
    private int projectileLevel;
    private int projectileBounceCount;
    private BigDecimal projectilePosX;
    private BigDecimal projectilePosY;
    private BigDecimal projectileVelX;
    private BigDecimal projectileVelY;

    public C2SPosition(Object[] incomingObject) {
        this(getLong(1, incomingObject), getBigDecimal(2, incomingObject), getBigDecimal(3, incomingObject), getBigDecimal(4, incomingObject), getBigDecimal(5, incomingObject), getBigDecimal(6, incomingObject), getBooleanFromInteger(6, incomingObject));
        if (incomingObject.length > 8) {
            this.projectileType = getInteger(8, incomingObject);
            this.projectileLevel = getInteger(9, incomingObject);
            this.projectileBounceCount = getInteger(10, incomingObject);
            this.projectilePosX = getBigDecimal(11, incomingObject);
            this.projectilePosY = getBigDecimal(12, incomingObject);
            this.projectileVelX = getBigDecimal(13, incomingObject);
            this.projectileVelY = getBigDecimal(14, incomingObject);
        }
    }

    public C2SPosition(long remoteTimeDelta, BigDecimal direction, BigDecimal posX, BigDecimal posY, BigDecimal velX, BigDecimal velY, boolean isSafe, int projectileType, int projectileLevel, int projectileBounceCount, BigDecimal projectilePosX, BigDecimal projectilePosY, BigDecimal projectileVelX, BigDecimal projectileVelY) {
        this(C2SPacketType.POSITION, remoteTimeDelta, direction, posX, posY, velX, velY, isSafe, projectileType, projectileLevel, projectileBounceCount, projectilePosX, projectilePosY, projectileVelX, projectileVelY);
    }

    public C2SPosition(long remoteTimeDelta, BigDecimal direction, BigDecimal posX, BigDecimal posY, BigDecimal velX, BigDecimal velY, boolean isSafe) {
        this(C2SPacketType.POSITION, remoteTimeDelta, direction, posX, posY, velX, velY, isSafe);
    }

    private C2SPosition(C2SPacketType packetType, long remoteTimeDelta, BigDecimal direction, BigDecimal posX, BigDecimal posY, BigDecimal velX, BigDecimal velY, boolean isSafe, int projectileType, int projectileLevel, int projectileBounceCount, BigDecimal projectilePosX, BigDecimal projectilePosY, BigDecimal projectileVelX, BigDecimal projectileVelY) {
        super(packetType);
        this.remoteTimeDelta = remoteTimeDelta;
        this.direction = direction;
        this.posX = posX;
        this.posY = posY;
        this.velX = velX;
        this.velY = velY;
        if(isSafe) {
            this.isSafe = 1;
        } else {
            this.isSafe = 0;
        }
        this.projectileType = projectileType;
        this.projectileLevel = projectileLevel;
        this.projectileBounceCount = projectileBounceCount;
        this.projectilePosX = projectilePosX;
        this.projectilePosY = projectilePosY;
        this.projectileVelX = projectileVelX;
        this.projectileVelY = projectileVelY;
    }

    private C2SPosition(C2SPacketType packetType, long remoteTimeDelta, BigDecimal direction, BigDecimal posX, BigDecimal posY, BigDecimal velX, BigDecimal velY, boolean isSafe) {
        super(packetType);
        this.remoteTimeDelta = remoteTimeDelta;
        this.direction = direction;
        this.posX = posX;
        this.posY = posY;
        this.velX = velX;
        this.velY = velY;
        if(isSafe) {
            this.isSafe = 1;
        } else {
            this.isSafe = 0;
        }
    }

    public long getRemoteTimeDelta() {
        return remoteTimeDelta;
    }

    public BigDecimal getDirection() {
        return direction;
    }

    public BigDecimal getPosX() {
        return posX;
    }

    public BigDecimal getPosY() {
        return posY;
    }

    public BigDecimal getVelX() {
        return velX;
    }

    public BigDecimal getVelY() {
        return velY;
    }

    public boolean isSafe() {
        if(this.isSafe == 1) {
            return true;
        } else {
            return false;
        }
    }

    public int getProjectileType() {
        return projectileType;
    }

    public int getProjectileLevel() {
        return projectileLevel;
    }

    public int getProjectileBounceCount() {
        return projectileBounceCount;
    }

    public BigDecimal getProjectilePosX() {
        return projectilePosX;
    }

    public BigDecimal getProjectilePosY() {
        return projectilePosY;
    }

    public BigDecimal getProjectileVelX() {
        return projectileVelX;
    }

    public BigDecimal getProjectileVelY() {
        return projectileVelY;
    }
}
