/*
 * Copyright (C) 2013 Christopher Burke <krslynx@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package org.xproduct.server.core.net.packets.core.s2c;

import com.google.gson.Gson;
import java.math.BigDecimal;
import org.xproduct.server.core.helpers.Converters;
import org.xproduct.server.core.net.packets.S2CPacketType;
import org.xproduct.server.core.net.packets.core.S2C;

/**
 *
 * @author Christopher Burke <krslynx@gmail.com>
 */
public class S2CPosition extends S2C {

    private long remoteTimeDelta;
    private BigDecimal direction;
    private String playerId;
    private BigDecimal posX;
    private BigDecimal posY;
    private BigDecimal velX;
    private BigDecimal velY;
    private boolean isSafe;
    private int projectileType;
    private int projectileLevel;
    private int projectileBounceCount;
    private BigDecimal projectilePosX;
    private BigDecimal projectilePosY;
    private BigDecimal projectileVelX;
    private BigDecimal projectileVelY;

    public S2CPosition(long remoteTimeDelta, BigDecimal direction, String playerId, BigDecimal posX, BigDecimal posY, BigDecimal velX, BigDecimal velY, boolean isSafe, int projectileType, int projectileLevel, int projectileBounceCount, BigDecimal projectilePosX, BigDecimal projectilePosY, BigDecimal projectileVelX, BigDecimal projectileVelY) {
        super(S2CPacketType.PLAYER_POSITION);
        this.remoteTimeDelta = remoteTimeDelta;
        this.direction = direction;
        this.playerId = playerId;
        this.posX = posX;
        this.posY = posY;
        this.velX = velX;
        this.velY = velY;
        this.isSafe = isSafe;
        this.projectileType = projectileType;
        this.projectileLevel = projectileLevel;
        this.projectileBounceCount = projectileBounceCount;
        this.projectilePosX = projectilePosX;
        this.projectilePosY = projectilePosY;
        this.projectileVelX = projectileVelX;
        this.projectileVelY = projectileVelY;
    }

    public S2CPosition(long remoteTimeDelta, BigDecimal direction, String playerId, BigDecimal posX, BigDecimal posY, BigDecimal velX, BigDecimal velY, boolean isSafe) {
        super(S2CPacketType.PLAYER_POSITION);
        this.remoteTimeDelta = remoteTimeDelta;
        this.direction = direction;
        this.playerId = playerId;
        this.posX = posX;
        this.posY = posY;
        this.velX = velX;
        this.velY = velY;
        this.isSafe = isSafe;
    }

    private S2CPosition(S2CPacketType packetType, long remoteTimeDelta, BigDecimal direction, String playerId, BigDecimal posX, BigDecimal posY, BigDecimal velX, BigDecimal velY, boolean isSafe, int projectileType, int projectileLevel, int projectileBounceCount, BigDecimal projectilePosX, BigDecimal projectilePosY, BigDecimal projectileVelX, BigDecimal projectileVelY) {
        super(packetType);
        this.remoteTimeDelta = remoteTimeDelta;
        this.direction = direction;
        this.playerId = playerId;
        this.posX = posX;
        this.posY = posY;
        this.velX = velX;
        this.velY = velY;
        this.isSafe = isSafe;
        this.projectileType = projectileType;
        this.projectileLevel = projectileLevel;
        this.projectileBounceCount = projectileBounceCount;
        this.projectilePosX = projectilePosX;
        this.projectilePosY = projectilePosY;
        this.projectileVelX = projectileVelX;
        this.projectileVelY = projectileVelY;
    }

    private S2CPosition(S2CPacketType packetType, long remoteTimeDelta, BigDecimal direction, String playerId, BigDecimal posX, BigDecimal posY, BigDecimal velX, BigDecimal velY, boolean isSafe) {
        super(packetType);
        this.remoteTimeDelta = remoteTimeDelta;
        this.direction = direction;
        this.playerId = playerId;
        this.posX = posX;
        this.posY = posY;
        this.velX = velX;
        this.velY = velY;
        this.isSafe = isSafe;
    }

    @Override
    public String getPacket() {

//        posX = new BigDecimal(posX.doubleValue() + (velX.doubleValue() * ((Converters.timeAsUInt32() - remoteTimeDelta) / 100.0)));
//        posY = new BigDecimal(posY.doubleValue() + (velY.doubleValue() * ((Converters.timeAsUInt32() - remoteTimeDelta) / 100.0)));

        if (projectilePosX == null) {
            return new Gson().toJson(new Object[]{
                        getPacketIdentifier(),
                        remoteTimeDelta,
                        playerId,
                        direction,
                        posX,
                        posY,
                        velX,
                        velY,
                        isSafe
                    });
        } else {

//            projectilePosX = new BigDecimal(projectilePosX.doubleValue() + (projectileVelX.doubleValue() * ((Converters.timeAsUInt32() - remoteTimeDelta) / 100.0)));
//            projectilePosY = new BigDecimal(projectilePosY.doubleValue() + (projectileVelY.doubleValue() * ((Converters.timeAsUInt32() - remoteTimeDelta) / 100.0)));

            return new Gson().toJson(new Object[]{
                        getPacketIdentifier(),
                        remoteTimeDelta,
                        playerId,
                        direction,
                        posX,
                        posY,
                        velX,
                        velY,
                        isSafe,
                        projectileType,
                        projectileLevel,
                        projectileBounceCount,
                        projectilePosX,
                        projectilePosY,
                        projectileVelX,
                        projectileVelY
                    });
        }

    }
}
